package sun.awt.X11;
public class XAWTIcon64_java_icon16_png {
public static long[] java_icon16_png = { 
16,16,
0x689fb5c9, 
0xcc9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 
0xff9fb5c8, 0xff9fb5c8, 0xff9fb5c8, 0xcc9fb5c8, 0x689fb5c9, 0xcc9bb2c4, 0xffd5dade, 0xfff5f5f5, 0xfff7f7f7, 0xfff7f7f7, 
0xfff7f7f7, 0xfff7f7f7, 0xff9b9b9b, 0xffbbbbbb, 0xfff4f4f4, 0xfff7f7f7, 0xfff7f7f7, 0xfff7f7f7, 0xfff5f5f5, 0xffd5dade, 
0xcc9bb2c4, 0xff96acbf, 0xffededed, 0xfffcfcfc, 0xfffdfdfd, 0xfffdfdfd, 0xfffdfdfd, 0xfffdfdfd, 0xff575757, 0xff454646, 
0xffb7a697, 0xfffafafa, 0xfffdfdfd, 0xfffdfdfd, 0xfffcfcfc, 0xffededed, 0xff96acbf, 0xff90a7ba, 0xffececec, 0xfffbfbfb, 
0xfffbfbfb, 0xfffbfbfb, 0xfffbfbfb, 0xffededed, 0xff2e3333, 0xff3c3e3e, 0xff42433f, 0xffcfd0d0, 0xfffbfbfb, 0xfffbfbfb, 
0xfffbfbfb, 0xffececec, 0xff90a7ba, 0xff89a1b5, 0xffe9e9e9, 0xfff9f9f9, 0xfff9f9f9, 0xfff9f9f9, 0xfff9f9f9, 0xff9e9f9f, 
0xff4f2828, 0xffd24747, 0xffab4a4a, 0xff5e564f, 0xfff5f5f5, 0xfff9f9f9, 0xfff9f9f9, 0xffe9e9e9, 0xff89a1b5, 0xff829baf, 
0xffe7e7e7, 0xfff6f6f6, 0xfff6f6f6, 0xffeae9e9, 0xffc6c6c6, 0xff413d35, 0xff7c2525, 0xfff84343, 0xfff84040, 0xff634241, 
0xffcecfcf, 0xfff4f4f4, 0xffececec, 0xffe7e7e7, 0xff829baf, 0xff7b94a8, 0xffe4e4e4, 0xffebebeb, 0xff999c9f, 0xff627381, 
0xff575c60, 0xff959696, 0xff462828, 0xff731515, 0xff7e1919, 0xffe3d2d2, 0xffa4a6a6, 0xffbebebe, 0xff949494, 0xffe4e4e4, 
0xff7b94a8, 0xff748da2, 0xffe1e1e1, 0xff878685, 0xff6b839b, 0xffb1d4f2, 0xff929ca6, 0xffacaba9, 0xff3a3b3b, 0xff070101, 
0xff4f4848, 0xffffffff, 0xffbbbbbb, 0xff5e5e5e, 0xff797979, 0xffe1e1e1, 0xff748da2, 0xff6c879b, 0xffdedede, 0xff5c5b5b, 
0xff413b33, 0xff504b43, 0xff8f867a, 0xffb1b2b3, 0xff7f8080, 0xff686a6a, 0xffced0d0, 0xffffffff, 0xffcbcdcf, 0xff1d1e1f, 
0xff444444, 0xffdcdcdc, 0xff6c879b, 0xff648094, 0xffdcdcdc, 0xff828282, 0xff484845, 0xff414e58, 0xff6f7274, 0xffbdbdbd, 
0xffc0c0c0, 0xffd5d5d5, 0xffffffff, 0xffffffff, 0xffd9d9d9, 0xff3c4f5e, 0xff56636e, 0xffd8d8d8, 0xff648094, 0xff5d798e, 
0xffd9d9d9, 0xffdedede, 0xffd7d7d7, 0xfffafafa, 0xff828282, 0xffaaaaaa, 0xffb7b7b7, 0xffd0d0d0, 0xffececec, 0xfffefefe, 
0xffe4e4e4, 0xff526a7d, 0xff899eaf, 0xffd5d5d5, 0xff5d798e, 0xff567287, 0xffd6d6d6, 0xffc6c6c6, 0xffa6a6a6, 0xff8f8f8f, 
0xff5b5c5e, 0xff737373, 0xff6a6a6a, 0xff797979, 0xff969696, 0xffd0d0d0, 0xffe1e1e1, 0xff71818e, 0xffe0e0e0, 0xffd4d4d4, 
0xff567287, 0xff4f6c81, 0xffdbdbdb, 0xffb4b4b4, 0xff919191, 0xff737373, 0xff414141, 0xff424242, 0xff1a1a1a, 0xff0f0f0f, 
0xff252525, 0xff7b7b7b, 0xffa1a1a1, 0xffbababa, 0xffe5e5e5, 0xffdbdbdb, 0xff4f6c81, 0xff48667c, 0xffdfdfdf, 0xffe6e6e6, 
0xffabbbc7, 0xff87a0b4, 0xff5f7a8f, 0xff536a7d, 0xff4f6576, 0xff485c6c, 0xff3e4f5d, 0xff384550, 0xff495b6a, 0xffe2e2e2, 
0xffe9e9e9, 0xffdfdfdf, 0xff48667c, 0xe1426177, 0xffaab0b5, 0xffe8e8e8, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffeaeaea, 0xffaab0b5, 0xe1426177, 0x683d5d71, 
0xcc3d5d72, 0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 
0xff3d5c72, 0xff3d5c72, 0xff3d5c72, 0xcc3d5d72, 0x683d5d71, }; }
