package sun.awt.X11;
public class XAWTIcon32_java_icon32_png {
public static int[] java_icon32_png = { 
32,32,
0x0, 
0x0, 0x409fb7c7, 0x7fa1b7c9, 0xbfa2b6ca, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 
0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 
0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xbfa2b6ca, 0x7fa1b7c9, 0x409fb7c7, 0x0, 
0x0, 0x0, 0x409fb3c7, 0xffaebecc, 0xffc0cbd5, 0xffd8dee2, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 
0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 
0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xffd8dee2, 0xffc0cbd5, 
0xffaebecc, 0x409fb3c7, 0x0, 0x409bb3c7, 0xffacbdcb, 0xffcfd7dd, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 
0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xffbab9be, 0xffa09fa4, 0xffeae9f0, 
0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 
0xfff4f3fa, 0xfff4f3fa, 0xffcfd7dd, 0xffacbdcb, 0x409bb3c7, 0x7f9bb1c5, 0xffbdc8d2, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 
0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xff8e8d91, 
0xff373737, 0xff858487, 0xffe6e6ec, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 
0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xffbdc8d2, 0x7f9bb1c5, 0xbf98aec2, 0xffd4d9de, 0xfff2f1f9, 
0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 
0xffecebf2, 0xff77777a, 0xff333333, 0xff494949, 0xff757578, 0xffdedde4, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 
0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xfff2f1f9, 0xffd4d9de, 0xbf98aec2, 0xff95abbf, 
0xffececec, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 
0xfff1f0f8, 0xfff1f0f8, 0xffd5d4db, 0xff555557, 0xff2a2a2a, 0xff444444, 0xff4c4c4c, 0xff69686b, 0xffd5d4db, 0xfff1f0f8, 
0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 
0xff95abbf, 0xff92a8bc, 0xffececec, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 
0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffb6b6bb, 0xff303031, 0xff292929, 0xff3a3a3a, 0xff4f4e4e, 0xff494949, 
0xff656567, 0xffd5d5db, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 
0xffefeff6, 0xffefeff6, 0xff92a8bc, 0xff8ea5b9, 0xffebebeb, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 
0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xff929296, 0xff222223, 0xff292929, 0xff303232, 
0xff414747, 0xff4d4f4f, 0xff424141, 0xff78777a, 0xffdfdfe6, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 
0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xff8ea5b9, 0xff8ba2b6, 0xffe9e9e9, 0xffececf4, 0xffececf4, 0xffececf4, 
0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffd5d5dc, 0xff5b5b5e, 0xff222525, 
0xff212828, 0xff3a2e2e, 0xff5c3e3e, 0xff534a4a, 0xff3e4646, 0xff363737, 0xff9a999e, 0xffebebf3, 0xffececf4, 0xffececf4, 
0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xff8ba2b6, 0xff879fb3, 0xffe8e8e8, 0xffebebf3, 
0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffa9a8ae, 
0xff292e2f, 0xff202626, 0xff6c2c2c, 0xffdb4040, 0xffff4d4d, 0xffdf4a4a, 0xff774040, 0xff353b3b, 0xff4d5153, 0xffc8c8cf, 
0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xff879fb3, 0xff849cb0, 
0xffe7e7e7, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 
0xffd8d8df, 0xff636568, 0xff182424, 0xff5a2626, 0xffdf3838, 0xffff4b4b, 0xffff5757, 0xffff5a5a, 0xffef4747, 0xff6f3636, 
0xff2c3535, 0xff8a8b8f, 0xffe5e5ed, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 
0xff849cb0, 0xff8098ac, 0xffe6e6e6, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 
0xffe7e7f0, 0xffe7e7f0, 0xffaeaeb4, 0xff212627, 0xff201e1e, 0xff752121, 0xffd93434, 0xfffa4242, 0xffff5050, 0xffff5252, 
0xffff4747, 0xffaa3333, 0xff2f2727, 0xff3b4143, 0xffcdcdd5, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 
0xffe7e7f0, 0xffe7e7f0, 0xff8098ac, 0xff7c95a9, 0xffe4e4e4, 0xffe6e6ee, 0xffe6e6ee, 0xffe6e6ee, 0xffe6e6ee, 0xffe5e5ed, 
0xffd4d4dc, 0xffbdbdc3, 0xffafafb4, 0xff919196, 0xff4f4f51, 0xff1c2222, 0xff221b1b, 0xff591616, 0xff9f2727, 0xffcc3434, 
0xffe63b3b, 0xffed3d3d, 0xffef3535, 0xffaa3030, 0xff635252, 0xff616868, 0xffb1b1b7, 0xffe6e6ee, 0xffe6e6ee, 0xffe0e0e8, 
0xffc4c4ca, 0xffdbdbe2, 0xffe6e6ee, 0xffe6e6ee, 0xff7c95a9, 0xff7891a6, 0xffe3e3e3, 0xffe4e4ed, 0xffe4e4ed, 0xffe4e4ed, 
0xffdddde6, 0xff89878a, 0xff4a4846, 0xff322e2c, 0xff1d1a1a, 0xff2d2d2e, 0xff787777, 0xffb1b4b4, 0xff706d6d, 0xff350f0f, 
0xff631313, 0xff952626, 0xffb02c2c, 0xffbc3131, 0xffa21515, 0xff923c3c, 0xffd2c6c6, 0xffffffff, 0xffa2a2a4, 0xffb9b9c0, 
0xffe4e4ed, 0xffc4c4cb, 0xff59595b, 0xff9f9fa5, 0xffe2e2eb, 0xffe4e4ed, 0xff7891a6, 0xff748ea2, 0xffe1e1e1, 0xffe2e2eb, 
0xffe0e0e8, 0xffa9a9af, 0xff5b5b5e, 0xff5a6b7c, 0xff6f8da9, 0xff6f8495, 0xff76797f, 0xff5b5a5c, 0xffa6a6a7, 0xffe8e9e9, 
0xff949595, 0xff231616, 0xff280101, 0xff571717, 0xff741e1e, 0xff7d2121, 0xff4e0000, 0xff7e5858, 0xffeceaea, 0xffffffff, 
0xffc8c8c8, 0xff525254, 0xffd1d1d9, 0xffc9c9d1, 0xff272728, 0xff838388, 0xffe0e0e8, 0xffe2e2eb, 0xff748ea2, 0xff708a9f, 
0xffe0e0e0, 0xffdadbe4, 0xff79797c, 0xff41474a, 0xff7d9ebd, 0xff9acdfb, 0xff96ceff, 0xff9ed4fc, 0xff93b4ca, 0xff858c93, 
0xffa7a3a1, 0xffc7c7c7, 0xff868787, 0xff2a2929, 0xff090000, 0xff1a0404, 0xff2d0b0b, 0xff220101, 0xff200b0b, 0xffa6a1a1, 
0xfffeffff, 0xffffffff, 0xffefefef, 0xff4e4e4e, 0xff9c9da3, 0xffd6d7df, 0xff333335, 0xff909196, 0xffdedfe8, 0xffe0e1ea, 
0xff708a9f, 0xff6c879b, 0xffdedede, 0xffcbccd4, 0xff424141, 0xff484f55, 0xff87add0, 0xff9bc5ef, 0xffabc9dc, 0xff9db6cc, 
0xff8492a7, 0xff909193, 0xffafadac, 0xffb7b7b7, 0xff858686, 0xff3d3f3f, 0xff1a1919, 0xff130e0e, 0xff0a0404, 0xff040000, 
0xff565858, 0xffe0e1e1, 0xffffffff, 0xffffffff, 0xffffffff, 0xff6f6f6f, 0xff66666a, 0xff999aa0, 0xff3f3f40, 0xff6f6f73, 
0xffd6d7df, 0xffdedfe8, 0xff6c879b, 0xff688397, 0xffdddddd, 0xffa3a4ab, 0xff222122, 0xff33302d, 0xff313435, 0xff636266, 
0xffccc2c0, 0xffc7bbb5, 0xff8c8074, 0xff968e84, 0xffb1b2b3, 0xffb2b2b3, 0xff919191, 0xff5a5a5a, 0xff313333, 0xff282828, 
0xff292929, 0xff515252, 0xffb6b6b6, 0xfffafafa, 0xffffffff, 0xffffffff, 0xffffffff, 0xff8d8d8d, 0xff2b2b2c, 0xff2b2b2c, 
0xff2e2e2e, 0xff3e3e3e, 0xffc4c5cd, 0xffdcdde6, 0xff688397, 0xff647f94, 0xffdcdcdc, 0xff8c8d93, 0xff242524, 0xff4c463b, 
0xff403426, 0xff191612, 0xff32312c, 0xff655d4c, 0xff7b6b53, 0xff9b9288, 0xffb2b4b7, 0xffb3b3b3, 0xffa3a3a3, 0xff878787, 
0xff686868, 0xff5f5f5f, 0xff757575, 0xffaeaeae, 0xffececec, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffafafb1, 
0xff424245, 0xff101010, 0xff1f1f1f, 0xff2c2c2d, 0xffbebfc7, 0xffdadbe5, 0xff647f94, 0xff607c90, 0xffdadada, 0xffa1a2a8, 
0xff191a1a, 0xff4f4a41, 0xff6f604c, 0xff26231c, 0xff000000, 0xff131511, 0xff5b5449, 0xff9a9793, 0xffb5b6b7, 0xffb8b8b8, 
0xffb3b3b3, 0xffacacac, 0xffa4a4a4, 0xffa6a6a6, 0xffb9b9b9, 0xffdfdfdf, 0xfffcfcfc, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffb6b6b8, 0xffa1a2a9, 0xff353537, 0xff1a1a1a, 0xff343435, 0xffc6c7cf, 0xffd9dae3, 0xff607c90, 0xff5c788d, 
0xffd9d9d9, 0xffc4c5ce, 0xff313233, 0xff2f2d2b, 0xff3a3935, 0xff353839, 0xff54585c, 0xff676a6f, 0xff606266, 0xff989798, 
0xffb7b7b7, 0xffb9b9b9, 0xffbcbcbc, 0xffbebebe, 0xffc1c1c1, 0xffc8c8c8, 0xffd8d8d8, 0xfff2f2f2, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffb1b1b1, 0xffacadb5, 0xffa0a1a8, 0xff252627, 0xff74757a, 0xffd1d2dc, 0xffd7d8e2, 
0xff5c788d, 0xff58758a, 0xffd7d7d7, 0xffd3d4de, 0xff88888f, 0xff1b1b1d, 0xff74767b, 0xffbabcc4, 0xffcfd0da, 0xffcccdd6, 
0xff818186, 0xff9d9d9d, 0xffb0b0b0, 0xffb7b7b7, 0xffbcbcbc, 0xffc1c1c1, 0xffc9c9c9, 0xffd4d4d4, 0xffe4e4e4, 0xfff7f7f7, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffb0b0b0, 0xff8b8c91, 0xffcdced8, 0xffb2b2bb, 0xffcbccd6, 
0xffd5d6e0, 0xffd5d6e0, 0xff58758a, 0xff547186, 0xffd6d6d6, 0xffd3d5df, 0xffcbcdd6, 0xffb1b3bb, 0xffd1d2dc, 0xffd3d5df, 
0xffd3d5df, 0xffc1c3cc, 0xff737477, 0xff989898, 0xffa6a6a6, 0xffadadad, 0xffb4b4b4, 0xffbababa, 0xffc3c3c3, 0xffcfcfcf, 
0xffdddddd, 0xffeaeaea, 0xfff6f6f6, 0xfffefefe, 0xffffffff, 0xffffffff, 0xffffffff, 0xffbbbbbb, 0xff8c8d91, 0xffcbcdd6, 
0xffd3d5df, 0xffd3d5df, 0xffd3d5df, 0xffd3d5df, 0xff547186, 0xff516e83, 0xffd6d6d6, 0xffd0d1db, 0xffd0d1da, 0xffd0d1db, 
0xffc9cad3, 0xffc2c2cc, 0xffc4c5ce, 0xffb5b6be, 0xff68696b, 0xff8b8b8b, 0xff969696, 0xff9b9b9b, 0xffa1a1a1, 0xffa8a8a8, 
0xffb1b1b1, 0xffbbbbbb, 0xffc6c6c6, 0xffd1d1d1, 0xffdddddd, 0xffeaeaea, 0xfff5f5f5, 0xfffefefe, 0xffffffff, 0xffcbcbcb, 
0xff939398, 0xffcdced7, 0xffd2d3dd, 0xffd2d3dd, 0xffd2d3dd, 0xffd2d3dd, 0xff516e83, 0xff4d6b80, 0xffd8d8d8, 0xffcbccd7, 
0xffc1c2cc, 0xffb7b8c1, 0xffadaeb7, 0xffa2a3ac, 0xff9d9ea7, 0xff909098, 0xff5c5c5f, 0xff7e7e7e, 0xff7e7e7e, 0xff7d7d7d, 
0xff808080, 0xff868686, 0xff8e8e8e, 0xff969696, 0xffa0a0a0, 0xffababab, 0xffbababa, 0xffc9c9c9, 0xffdadada, 0xffebebeb, 
0xfff9f9f9, 0xffcbcbcb, 0xff9c9da2, 0xffcccdd7, 0xffd0d1dc, 0xffd0d1dc, 0xffd0d1dc, 0xffd0d1dc, 0xff4d6b80, 0xff4a687d, 
0xffd8d8d8, 0xffbcbec8, 0xffaaacb5, 0xff999ba3, 0xff898a91, 0xff76777d, 0xff696a70, 0xff5d5e62, 0xff4a4b4d, 0xff767676, 
0xff6d6d6d, 0xff585858, 0xff525252, 0xff535353, 0xff5a5a5a, 0xff616161, 0xff696969, 0xff747474, 0xff858585, 0xff9d9d9d, 
0xffbababa, 0xffd6d6d6, 0xffececec, 0xffcacaca, 0xff999a9f, 0xffcaccd7, 0xffced0db, 0xffced0db, 0xffced0db, 0xffced0db, 
0xff4a687d, 0xff46657a, 0xffd7d7d7, 0xffabadb6, 0xff909199, 0xff7d7e85, 0xff68696e, 0xff4f5054, 0xff404144, 0xff333437, 
0xff2e2e30, 0xff666666, 0xff717171, 0xff4c4c4c, 0xff373737, 0xff303030, 0xff2f2f2f, 0xff2f3030, 0xff343435, 0xff3c3d3d, 
0xff4b4b4b, 0xff686868, 0xff939393, 0xffc0c0c0, 0xffe1e1e1, 0xffbebebe, 0xff85878d, 0xffc7c9d4, 0xffcdcfda, 0xffcdcfda, 
0xffcdcfda, 0xffcdcfda, 0xff46657a, 0xbf436177, 0xffafb7bc, 0xffb2b2bc, 0xff9d9ea6, 0xff8e8f97, 0xff87888f, 0xff7e7f86, 
0xff76777d, 0xff626267, 0xff333336, 0xff2d2d2d, 0xff565656, 0xff484848, 0xff2d2d2d, 0xff1e1e1f, 0xff2a2a2c, 0xff393a3d, 
0xff39393c, 0xff2a2b2c, 0xff262626, 0xff3c3c3c, 0xff686868, 0xffa1a1a1, 0xffc4c4c4, 0xff878788, 0xff96979f, 0xffc0c1cb, 
0xffc2c3cd, 0xffc9cad5, 0xffcccdd8, 0xffafb7bc, 0xbf436177, 0x7f405e74, 0xff8797a2, 0xffc7c8d3, 0xffc4c5cf, 0xffc2c3ce, 
0xffc4c5cf, 0xffc5c6d1, 0xffc1c2cc, 0xffb2b3bd, 0xff8d8e95, 0xff56575b, 0xff434346, 0xff464649, 0xff424246, 0xff4f4f53, 
0xff6f7076, 0xff8a8a92, 0xff85868d, 0xff5f5f64, 0xff2d2d2f, 0xff181818, 0xff343434, 0xff5c5c5c, 0xff626262, 0xff606064, 
0xffafb0ba, 0xffb9bac4, 0xffbbbcc6, 0xffc5c6d0, 0xffcbccd7, 0xff8797a2, 0x7f405e74, 0x40405c74, 0xff4f6676, 0xffa5afb6, 
0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 0xffc7c9d5, 0xffc6c8d4, 0xffc0c2ce, 0xffb9bbc6, 0xffb5b7c2, 
0xffb4b6c0, 0xffb5b7c2, 0xffbcbeca, 0xffc3c5d1, 0xffc2c4cf, 0xffb5b7c2, 0xff9b9da6, 0xff76777e, 0xff56565b, 0xff4d4e52, 
0xff6c6e74, 0xffaeb0ba, 0xffc5c7d3, 0xffc5c7d3, 0xffc7c9d4, 0xffc8cad6, 0xffa5afb6, 0xff4f6676, 0x40405c74, 0x0, 
0x403c5c70, 0xff4e6676, 0xff768590, 0xffacb4b9, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 
0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 
0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffc8cad6, 0xffacb4b9, 0xff768590, 0xff4e6676, 0x403c5c70, 
0x0, 0x0, 0x0, 0x40385870, 0x7f38586e, 0xbf39586d, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 
0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 
0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xff39586e, 0xbf39586d, 0x7f38586e, 
0x40385870, 0x0, 0x0, }; }
