package sun.awt.X11;
public class XAWTIcon32_security_icon_yellow16_png {
public static int[] security_icon_yellow16_png = { 
16,16,
0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffd8b600, 0xffd8b600, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xffd6b400, 0xffedd459, 0xffedd459, 0xffd6b400, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffd3b100, 0xffffe563, 0xffffe563, 
0xffd3b100, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0xffcfae00, 0xffedd146, 0xffffe357, 0xffffe357, 0xffedd146, 0xffcfae00, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffcbab00, 0xffffe24b, 
0xff444339, 0xff514d3a, 0xffffe24b, 0xffcbab00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xffc7a700, 0xffedce32, 0xffffdf3e, 0xff272727, 0xff353329, 0xffffdf3e, 0xffedce32, 
0xffc7a700, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffc3a300, 
0xffffdd32, 0xffffdd32, 0xff1a1a1a, 0xff534b20, 0xffffdd32, 0xffffdd32, 0xffc3a300, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0xffbe9f00, 0xffedca1e, 0xffffdb26, 0xffffdb26, 0xff2d2912, 0xff4a4114, 
0xffffdb26, 0xffffdb26, 0xffedca1e, 0xffbe9f00, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0xffb99b00, 0xffffdd27, 0xffffdd27, 0xffffdd27, 0xff443c0e, 0xff827116, 0xffffdd27, 0xffffdd27, 0xffffdd27, 0xffb99b00, 
0x0, 0x0, 0x0, 0x0, 0x0, 0xffb49700, 0xffedcf22, 0xffffe12b, 0xffffe12b, 0xffffe12b, 
0xff51480f, 0xff807116, 0xffffe12b, 0xffffe12b, 0xffffe12b, 0xffedcf22, 0xffb49700, 0x0, 0x0, 0x0, 
0x0, 0xffaf9300, 0xffffe530, 0xffffe530, 0xffffe530, 0xffffe530, 0xffcfba27, 0xffefd72d, 0xffffe530, 0xffffe530, 
0xffffe530, 0xffffe530, 0xffaf9300, 0x0, 0x0, 0x0, 0xffab8f00, 0xffedd62c, 0xffffea37, 0xffffea37, 
0xffffea37, 0xffffea37, 0xff8d8435, 0xffa69a36, 0xffffea37, 0xffffea37, 0xffffea37, 0xffffea37, 0xffedd62c, 0xffab8f00, 
0x0, 0x0, 0xffa78b00, 0xffffef3d, 0xffffef3d, 0xffffef3d, 0xffffef3d, 0xffffef3d, 0xff1c1b10, 0xff59541c, 
0xffffef3d, 0xffffef3d, 0xffffef3d, 0xffffef3d, 0xffffef3d, 0xffa78b00, 0x0, 0xffa38800, 0xfffff544, 0xfffff544, 
0xfffff544, 0xfffff544, 0xfffff544, 0xfffff544, 0xfffff544, 0xfffff544, 0xfffff544, 0xfffff544, 0xfffff544, 0xfffff544, 
0xfffff544, 0xfffff544, 0xffa38800, 0xff9f8500, 0xffdbd031, 0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 
0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 0xfffffb4c, 0xffdbd031, 0xff9f8500, 0x0, 
0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 
0xff9c8200, 0xff9c8200, 0xff9c8200, 0xff9c8200, 0x0, }; }
