package sun.awt.X11;
public class XAWTIcon64_security_icon_bw48_png {
public static long[] security_icon_bw48_png = { 
48,48,
0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xffa1a1a1, 0xffa1a1a1, 0xffa1a1a1, 0xffa1a1a1, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffa0a0a0, 0xffcccccc, 
0xffe3e3e3, 0xffe3e3e3, 0xffcccccc, 0xffa0a0a0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xffa0a0a0, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 
0xffe2e2e2, 0xffa0a0a0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff9f9f9f, 0xffcbcbcb, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffcbcbcb, 
0xff9f9f9f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff9e9e9e, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xff9e9e9e, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff9e9e9e, 0xffcacaca, 
0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffcacaca, 0xff9e9e9e, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff9d9d9d, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 
0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xff9d9d9d, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff9c9c9c, 0xffc9c9c9, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 
0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffc9c9c9, 0xff9c9c9c, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff9b9b9b, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 
0xffdfdfdf, 0xffdfdfdf, 0xff9b9b9b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff9a9a9a, 0xffc8c8c8, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffc8c8c8, 0xff9a9a9a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff999999, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xff999999, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff989898, 0xffc7c7c7, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffbfbfbf, 0xff656565, 
0xff3e3e3e, 0xff3e3e3e, 0xff656565, 0xffbfbfbf, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffc7c7c7, 0xff989898, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff979797, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xff616161, 0xff383838, 0xff383838, 0xff383838, 
0xff383838, 0xff616161, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xff979797, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff969696, 0xffc6c6c6, 
0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xff333333, 0xff333333, 0xff333333, 0xff333333, 0xff333333, 0xff333333, 
0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffc6c6c6, 0xff969696, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff959595, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 
0xffdcdcdc, 0xffdcdcdc, 0xff2e2e2e, 0xff2e2e2e, 0xff2e2e2e, 0xff2e2e2e, 0xff2e2e2e, 0xff2e2e2e, 0xffdcdcdc, 0xffdcdcdc, 
0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xff959595, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff939393, 0xffc6c6c6, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xff292929, 0xff292929, 0xff292929, 0xff292929, 0xff292929, 0xff292929, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xffdbdbdb, 0xffc6c6c6, 0xff939393, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff929292, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xff303030, 0xff252525, 
0xff252525, 0xff252525, 0xff252525, 0xff303030, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xff929292, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff919191, 0xffc5c5c5, 
0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xff4f4f4f, 0xff212121, 0xff212121, 0xff212121, 
0xff212121, 0xff4f4f4f, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffc5c5c5, 0xff919191, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff909090, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 
0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xff4c4c4c, 0xff1d1d1d, 0xff1d1d1d, 0xff1d1d1d, 0xff1d1d1d, 0xff4c4c4c, 
0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xff909090, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff8e8e8e, 0xffc4c4c4, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 
0xffd9d9d9, 0xffd9d9d9, 0xff6d6d6d, 0xff191919, 0xff191919, 0xff191919, 0xff191919, 0xff6d6d6d, 0xffd9d9d9, 0xffd9d9d9, 
0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffc4c4c4, 0xff8e8e8e, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff8d8d8d, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xff767676, 0xff151515, 0xff151515, 0xff151515, 0xff151515, 0xff767676, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xff8d8d8d, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff8c8c8c, 0xffc3c3c3, 
0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xff818181, 0xff121212, 
0xff121212, 0xff121212, 0xff121212, 0xff818181, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xffd8d8d8, 0xffd8d8d8, 0xffc3c3c3, 0xff8c8c8c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff8b8b8b, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 
0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffa5a5a5, 0xff0f0f0f, 0xff0f0f0f, 0xff0f0f0f, 
0xff0f0f0f, 0xffa5a5a5, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 
0xffd7d7d7, 0xff8b8b8b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff898989, 0xffc2c2c2, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 
0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffa4a4a4, 0xff0c0c0c, 0xff0c0c0c, 0xff0c0c0c, 0xff0c0c0c, 0xffa4a4a4, 
0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffc2c2c2, 
0xff898989, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff888888, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 
0xffd6d6d6, 0xffd6d6d6, 0xffc9c9c9, 0xff0a0a0a, 0xff0a0a0a, 0xff0a0a0a, 0xff0a0a0a, 0xffc9c9c9, 0xffd6d6d6, 0xffd6d6d6, 
0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xff888888, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff878787, 0xffc2c2c2, 
0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 
0xffd6d6d6, 0xff080808, 0xff080808, 0xff080808, 0xff080808, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 
0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffc2c2c2, 0xff878787, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff858585, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 
0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xff131313, 
0xff060606, 0xff060606, 0xff131313, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 
0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xff858585, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff848484, 0xffc3c3c3, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xff393939, 0xff040404, 0xff040404, 
0xff393939, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffc3c3c3, 0xff848484, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff838383, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 
0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xff393939, 0xff030303, 0xff030303, 0xff393939, 0xffd9d9d9, 
0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 
0xffd9d9d9, 0xffd9d9d9, 0xff838383, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff828282, 0xffc5c5c5, 
0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 
0xffdadada, 0xffdadada, 0xffdadada, 0xff616161, 0xff020202, 0xff020202, 0xff616161, 0xffdadada, 0xffdadada, 0xffdadada, 
0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 
0xffc5c5c5, 0xff828282, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff808080, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xffdbdbdb, 0xff6e6e6e, 0xff010101, 0xff010101, 0xff6e6e6e, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xff808080, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff7f7f7f, 0xffc6c6c6, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 
0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xff7b7b7b, 
0xff000000, 0xff000000, 0xff7b7b7b, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 
0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffc6c6c6, 0xff7f7f7f, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff7e7e7e, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 
0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffcfcfcf, 0xff454545, 0xff454545, 
0xffcfcfcf, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 
0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xff7e7e7e, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff7d7d7d, 0xffc8c8c8, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffc8c8c8, 0xff7d7d7d, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff7b7b7b, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 
0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 
0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 
0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 
0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xff7b7b7b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff7a7a7a, 0xffcacaca, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 
0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 
0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 
0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 
0xffe1e1e1, 0xffcacaca, 0xff7a7a7a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff797979, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 
0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffcdcdcd, 
0xff909090, 0xff909090, 0xffcdcdcd, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 
0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 0xffe2e2e2, 
0xff797979, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff787878, 0xffcdcdcd, 
0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 
0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffc1c1c1, 0xff393939, 0xff2f2f2f, 0xff2f2f2f, 
0xff393939, 0xffc1c1c1, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 
0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffcdcdcd, 0xff787878, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff777777, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 
0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 
0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xff5c5c5c, 0xff1d1d1d, 0xff1d1d1d, 0xff1d1d1d, 0xff1d1d1d, 0xff5c5c5c, 
0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 
0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xff777777, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff767676, 0xffcfcfcf, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xff464646, 0xff101010, 0xff101010, 0xff101010, 0xff101010, 0xff464646, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffcfcfcf, 0xff767676, 0x0, 0x0, 0x0, 
0x0, 0xff757575, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 
0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 
0xff777777, 0xff060606, 0xff060606, 0xff060606, 0xff060606, 0xff777777, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 
0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 
0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xffe8e8e8, 0xff757575, 0x0, 0x0, 0x0, 0xff747474, 0xffd2d2d2, 
0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 
0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffdbdbdb, 0xff3b3b3b, 
0xff010101, 0xff010101, 0xff3b3b3b, 0xffdbdbdb, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 
0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 
0xffeaeaea, 0xffeaeaea, 0xffd2d2d2, 0xff747474, 0x0, 0x0, 0xff737373, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xff737373, 0x0, 0xff727272, 0xffd4d4d4, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 
0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 
0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 
0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 
0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffededed, 0xffd4d4d4, 
0xff727272, 0xff727272, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 
0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 
0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 
0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 
0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xff727272, 0xff717171, 
0xffd7d7d7, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xffd7d7d7, 0xff717171, 0xff707070, 0xffb2b2b2, 0xffd8d8d8, 
0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 
0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 
0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 
0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 0xfff2f2f2, 
0xfff2f2f2, 0xfff2f2f2, 0xffd8d8d8, 0xffb2b2b2, 0xff707070, 0x0, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0x0, }; }
