package sun.util.resources;

import java.util.ListResourceBundle;

public final class CurrencyNames_da_DK extends LocaleNamesBundle {
    protected final Object[][] getContents() {
        return new Object[][] {
            { "DKK", "kr" },
            { "EUR", "\u20AC" },
        };
    }
}
