package sun.awt.X11;
public class XAWTIcon64_java_icon24_png {
public static long[] java_icon24_png = { 
24,23,
0x1a9dbac4, 
0x80a1b5c9, 0xd5a0b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 
0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 0xffa1b6c9, 
0xd5a0b6c9, 0x80a1b5c9, 0x1a9dbac4, 0x809db3c5, 0xffbec9d2, 0xffe8ebed, 0xfff8f8f8, 0xfffafafa, 0xfffafafa, 0xfffafafa, 
0xfffafafa, 0xfffafafa, 0xfffafafa, 0xffe3e3e3, 0xff989898, 0xffdadada, 0xfffafafa, 0xfffafafa, 0xfffafafa, 0xfffafafa, 
0xfffafafa, 0xfffafafa, 0xfffafafa, 0xfff8f8f8, 0xffe8ebed, 0xffbec9d2, 0x809db3c5, 0xd59ab0c3, 0xffdce0e3, 0xfff5f4fb, 
0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xffd2d1d7, 0xff4c4c4d, 0xff959598, 
0xfff3f2f9, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xffdce0e3, 
0xd59ab0c3, 0xff96adc0, 0xffededed, 0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 
0xfff3f3fa, 0xffbbbbc1, 0xff2b2b2b, 0xff4d4d4d, 0xff88888b, 0xffe9e9f0, 0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 
0xfff3f3fa, 0xfff3f3fa, 0xfff3f3fa, 0xffededed, 0xff96adc0, 0xff92a9bc, 0xffececec, 0xfff2f1f8, 0xfff2f1f8, 0xfff2f1f8, 
0xfff2f1f8, 0xfff2f1f8, 0xfff2f1f8, 0xfff2f1f8, 0xffeeedf4, 0xff8e8e92, 0xff282626, 0xff414242, 0xff4c4f4f, 0xff807f82, 
0xffe6e5ec, 0xfff2f1f8, 0xfff2f1f8, 0xfff2f1f8, 0xfff2f1f8, 0xfff2f1f8, 0xfff2f1f8, 0xffececec, 0xff92a9bc, 0xff8ea5b9, 
0xffebebeb, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xffe5e4ec, 0xff4c4c4e, 
0xff1e2828, 0xff293434, 0xff424c4c, 0xff3d4748, 0xff8a8a8e, 0xffeeedf5, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 
0xfff0eff7, 0xffebebeb, 0xff8ea5b9, 0xff89a1b4, 0xffe9e9e9, 0xffededf5, 0xffededf5, 0xffededf5, 0xffededf5, 0xffededf5, 
0xffededf5, 0xffededf5, 0xffafb0b5, 0xff232627, 0xff4b2a2a, 0xffb53e3e, 0xffc34c4c, 0xff704646, 0xff394243, 0xffaeafb5, 
0xffededf5, 0xffededf5, 0xffededf5, 0xffededf5, 0xffededf5, 0xffe9e9e9, 0xff89a1b4, 0xff849cb0, 0xffe7e7e7, 0xffebebf3, 
0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffe5e5ed, 0xff525558, 0xff342323, 0xffba3232, 0xfffe4e4e, 
0xffff5858, 0xffdf4949, 0xff623636, 0xff5e6568, 0xffdedee6, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffe7e7e7, 
0xff849cb0, 0xff7f98ac, 0xffe5e5e5, 0xffe8e8f1, 0xffe8e8f1, 0xffe8e8f1, 0xffe1e1e9, 0xffd2d1d9, 0xffcacad2, 0xff97989d, 
0xff282c2d, 0xff3d1b1b, 0xffad2828, 0xffea3f3f, 0xfffc4848, 0xfff43c3c, 0xff8c3939, 0xff454e4f, 0xffb5b6bc, 0xffe8e8f1, 
0xffe4e4ec, 0xffd9d9e1, 0xffe4e4ed, 0xffe5e5e5, 0xff7f98ac, 0xff7a93a7, 0xffe3e3e3, 0xffe6e6ee, 0xffe1e1e8, 0xffc3c3ca, 
0xff7f8286, 0xff525456, 0xff343435, 0xff565656, 0xff8d8f8f, 0xff4e3d3d, 0xff540e0e, 0xff9d2828, 0xffbb2a2a, 0xff9c1313, 
0xffa86e6e, 0xffeef3f3, 0xffa0a1a4, 0xffd6d6de, 0xffc7c7ce, 0xff6c6c6f, 0xffc4c4cb, 0xffe3e3e3, 0xff7a93a7, 0xff748ea2, 
0xffe2e2e2, 0xffd0d0d8, 0xff89898e, 0xff687b8d, 0xff7ca5c2, 0xff749bb9, 0xff64717d, 0xff8b8b8a, 0xffd4d5d5, 0xff666464, 
0xff130000, 0xff410e0e, 0xff480a0a, 0xff400d0d, 0xffb0a3a3, 0xffffffff, 0xffbfbfbf, 0xff848489, 0xffbebec5, 0xff434345, 
0xffb1b1b8, 0xffe2e2e2, 0xff748ea2, 0xff6f899d, 0xffe0e0e0, 0xff7a797c, 0xff393f43, 0xff82a6ca, 0xffbae3fc, 0xffbcdffe, 
0xff94a3b5, 0xffa2a1a0, 0xffadaeac, 0xff636464, 0xff121313, 0xff050101, 0xff000202, 0xff4f5353, 0xffe4e7e7, 0xffffffff, 
0xffe3e3e3, 0xff545455, 0xff828287, 0xff49494b, 0xffa3a3a9, 0xffe0e0e0, 0xff6f899d, 0xff698499, 0xffdddddd, 0xff4c4c4f, 
0xff383834, 0xff3f3f40, 0xff787777, 0xffa1958b, 0xff918477, 0xffaba8a6, 0xffacaeaf, 0xff828282, 0xff464747, 0xff2d3030, 
0xff4d5050, 0xffc0c2c2, 0xffffffff, 0xffffffff, 0xfffbfbfb, 0xff616162, 0xff19191a, 0xff303030, 0xff808085, 0xffdddddd, 
0xff698499, 0xff647f94, 0xffdbdbdb, 0xff46484c, 0xff463f37, 0xff453a29, 0xff0e0b07, 0xff2b261a, 0xff756c5e, 0xffadacaa, 
0xffb5b7b7, 0xffa9a9a9, 0xff949494, 0xff929292, 0xffc3c3c3, 0xfffbfbfb, 0xffffffff, 0xffffffff, 0xffffffff, 0xff8f8f93, 
0xff313234, 0xff1a1a1a, 0xff7c7d81, 0xffdbdbdb, 0xff647f94, 0xff5e7a8f, 0xffdadada, 0xff76767c, 0xff282623, 0xff474744, 
0xff5a5b5f, 0xff6b6c72, 0xff6f7071, 0xffafb0b0, 0xffbbbbbb, 0xffbfbfbf, 0xffc4c4c4, 0xffd1d0d0, 0xffeeeeee, 0xffffffff, 
0xffffffff, 0xffffffff, 0xfffbfbfb, 0xff959599, 0xffa2a3aa, 0xff56575a, 0xffabacb4, 0xffdadada, 0xff5e7a8f, 0xff59758a, 
0xffd8d8d8, 0xffc1c2cb, 0xff7d7e84, 0xffa5a6ae, 0xffd4d5df, 0xffcdced8, 0xff7c7d80, 0xffaaaaaa, 0xffb5b5b5, 0xffbdbdbd, 
0xffc9c9c9, 0xffdadada, 0xffeeeeee, 0xfffdfdfd, 0xffffffff, 0xffffffff, 0xfffdfdfd, 0xff8c8c8e, 0xffc0c1ca, 0xffcdcdd7, 
0xffd4d5df, 0xffd8d8d8, 0xff59758a, 0xff547186, 0xffd6d6d6, 0xffd1d3dd, 0xffd1d3dd, 0xffcfd1db, 0xffcfd1da, 0xffb6b7c0, 
0xff717173, 0xff979797, 0xff9f9f9f, 0xffa8a8a8, 0xffb4b4b4, 0xffc2c2c2, 0xffd2d2d2, 0xffe3e3e3, 0xfff1f1f1, 0xfffdfdfd, 
0xffffffff, 0xff979798, 0xffc1c3cb, 0xffd1d3dd, 0xffd1d3dd, 0xffd6d6d6, 0xff547186, 0xff4f6c82, 0xffd7d7d7, 0xffc0c1cb, 
0xffb3b3bd, 0xff9c9da5, 0xff8f9097, 0xff78787f, 0xff606162, 0xff7d7d7d, 0xff6e6e6e, 0xff6e6e6e, 0xff787878, 0xff868686, 
0xff929292, 0xffa6a6a6, 0xffc1c1c1, 0xffe1e1e1, 0xfff7f7f7, 0xff9d9d9f, 0xffc6c7d0, 0xffcfd0db, 0xffcfd0db, 0xffd7d7d7, 
0xff4f6c82, 0xff4a687d, 0xffd9d9d9, 0xff9d9ea6, 0xff84848c, 0xff69696f, 0xff505155, 0xff3a3a3d, 0xff3f4040, 0xff717171, 
0xff535353, 0xff393939, 0xff363637, 0xff3c3c3c, 0xff434343, 0xff575757, 0xff7f7f7f, 0xffbebebe, 0xffebebeb, 0xff919294, 
0xffbbbcc6, 0xffcdced9, 0xffcdced9, 0xffd9d9d9, 0xff4a687d, 0xff46647a, 0xffdcdcdc, 0xffa3a4ae, 0xff9b9ca5, 0xff9798a1, 
0xff8f9098, 0xff7a7a82, 0xff3e3e42, 0xff363637, 0xff373737, 0xff272728, 0xff333336, 0xff434347, 0xff343537, 0xff1e1e1f, 
0xff3c3c3c, 0xff878787, 0xffa7a7a7, 0xff76767b, 0xffb9bac5, 0xffc3c4d0, 0xffc9cad6, 0xffdcdcdc, 0xff46647a, 0xd5426077, 
0xffc2c7ca, 0xffc4c6d1, 0xffc5c7d2, 0xffc7c9d4, 0xffc7c9d4, 0xffc3c5d0, 0xffacaeb7, 0xff909199, 0xff8b8c93, 0xff8c8e95, 
0xff9799a1, 0xffa2a3ac, 0xff93949c, 0xff67686e, 0xff444548, 0xff454648, 0xff5e5f63, 0xffa5a7af, 0xffc3c5cf, 0xffc3c5cf, 
0xffc8cad5, 0xffc2c7ca, 0xd5426077, 0x803e5e74, 0xff778792, 0xffbbc1c4, 0xffe6e6e6, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe6e6e6, 0xffbbc1c4, 0xff778792, 0x803e5e74, 0x1a3b586c, 0x803a5a70, 0xd5395a71, 
0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 
0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xff3a5a70, 0xd5395a71, 0x803a5a70, 
0x1a3b586c, }; }
