package sun.awt.X11;
public class XAWTIcon32_security_icon_bw32_png {
public static int[] security_icon_bw32_png = { 
32,32,
0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0xffa1a1a1, 0xffa1a1a1, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0xffa0a0a0, 0xffcccccc, 0xffcccccc, 0xffa0a0a0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff9f9f9f, 0xffe2e2e2, 0xffe2e2e2, 
0xff9f9f9f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff9e9e9e, 0xffcacaca, 
0xffe1e1e1, 0xffe1e1e1, 0xffcacaca, 0xff9e9e9e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0xff9d9d9d, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xffe0e0e0, 0xff9d9d9d, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff9b9b9b, 0xffc9c9c9, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffc9c9c9, 0xff9b9b9b, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff9a9a9a, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 
0xff9a9a9a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0xff989898, 0xffc8c8c8, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffc8c8c8, 0xff989898, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff979797, 0xffdddddd, 0xffdddddd, 0xffdddddd, 
0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xff979797, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff959595, 0xffc6c6c6, 0xffdcdcdc, 
0xffdcdcdc, 0xff8b8b8b, 0xff3c3c3c, 0xff3c3c3c, 0xff8b8b8b, 0xffdcdcdc, 0xffdcdcdc, 0xffc6c6c6, 0xff959595, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff939393, 
0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xff333333, 0xff333333, 0xff333333, 0xff333333, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xff939393, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0xff929292, 0xffc5c5c5, 0xffdadada, 0xffdadada, 0xffdadada, 0xff2b2b2b, 0xff2b2b2b, 0xff2b2b2b, 0xff2b2b2b, 0xffdadada, 
0xffdadada, 0xffdadada, 0xffc5c5c5, 0xff929292, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0xff909090, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xff3b3b3b, 0xff242424, 0xff242424, 
0xff3b3b3b, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xff909090, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xff8e8e8e, 0xffc4c4c4, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xff4d4d4d, 
0xff1e1e1e, 0xff1e1e1e, 0xff4d4d4d, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffd9d9d9, 0xffc4c4c4, 0xff8e8e8e, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0xff8c8c8c, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xffd8d8d8, 0xff545454, 0xff171717, 0xff171717, 0xff545454, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xff8c8c8c, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff8a8a8a, 0xffc2c2c2, 0xffd7d7d7, 0xffd7d7d7, 
0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xff747474, 0xff121212, 0xff121212, 0xff747474, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 
0xffd7d7d7, 0xffd7d7d7, 0xffc2c2c2, 0xff8a8a8a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff888888, 0xffd6d6d6, 
0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xff727272, 0xff0e0e0e, 0xff0e0e0e, 0xff727272, 0xffd6d6d6, 
0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xff888888, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff868686, 
0xffc2c2c2, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xff979797, 0xff0a0a0a, 0xff0a0a0a, 
0xff979797, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffc2c2c2, 0xff868686, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff848484, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffa3a3a3, 
0xff060606, 0xff060606, 0xffa3a3a3, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xff848484, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0xff828282, 0xffc5c5c5, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 
0xffdadada, 0xffb2b2b2, 0xff030303, 0xff030303, 0xffb2b2b2, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 
0xffdadada, 0xffdadada, 0xffc5c5c5, 0xff828282, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0xff808080, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xff020202, 0xff020202, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 
0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xffdbdbdb, 0xff808080, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0xff7e7e7e, 0xffc7c7c7, 0xffdddddd, 0xffdddddd, 0xffdddddd, 
0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xff0e0e0e, 0xff0e0e0e, 0xffdddddd, 0xffdddddd, 
0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffdddddd, 0xffc7c7c7, 0xff7e7e7e, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff7d7d7d, 0xffdfdfdf, 0xffdfdfdf, 
0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffa7a7a7, 0xffa7a7a7, 
0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 
0xff7d7d7d, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff7b7b7b, 0xffcacaca, 
0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 
0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 
0xffe1e1e1, 0xffe1e1e1, 0xffcacaca, 0xff7b7b7b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0xff797979, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 
0xffe3e3e3, 0xffe3e3e3, 0xffa5a5a5, 0xffa5a5a5, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 
0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xffe3e3e3, 0xff797979, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff787878, 0xffcecece, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 
0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xff6f6f6f, 0xff292929, 0xff292929, 0xff6f6f6f, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 
0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffe5e5e5, 0xffcecece, 0xff787878, 0x0, 
0x0, 0x0, 0x0, 0xff767676, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xff474747, 0xff111111, 0xff111111, 0xff474747, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 
0xff767676, 0x0, 0x0, 0x0, 0xff757575, 0xffd2d2d2, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 
0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xff939393, 0xff030303, 0xff030303, 
0xff939393, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 
0xffeaeaea, 0xffeaeaea, 0xffd2d2d2, 0xff757575, 0x0, 0x0, 0xff737373, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 
0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xffececec, 0xff737373, 0x0, 0xff727272, 0xffd6d6d6, 0xffefefef, 
0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 
0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 
0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffefefef, 0xffd6d6d6, 0xff727272, 0xff717171, 
0xffc2c2c2, 0xffd7d7d7, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xffd7d7d7, 0xffc2c2c2, 
0xff717171, 0x0, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0x0, }; }
