package sun.util.resources;

import java.util.ListResourceBundle;

public final class CurrencyNames_fr_LU extends LocaleNamesBundle {
    protected final Object[][] getContents() {
        return new Object[][] {
            { "EUR", "\u20AC" },
            { "LUF", "F" },
        };
    }
}
