package sun.awt.X11;
public class XAWTIcon64_java_icon48_png {
public static long[] java_icon48_png = { 
48,48,
0x0, 
0x0, 0x33a0b9c8, 0x96a1b8ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 
0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 
0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 
0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 
0xffa2b7ca, 0xffa2b7ca, 0xffa2b7ca, 0x9fa2b7ca, 0x4ba3b8c9, 0x0, 0x0, 0x0, 0x66a0b7c8, 0xfca1b6c9, 
0xffcad3da, 0xfff0f0f2, 0xfff4f3f6, 0xfff6f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 
0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 
0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 
0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff7f6f9, 0xfff6f6f9, 0xfff4f4f7, 
0xfff1f1f3, 0xffced6dd, 0xffb3c1cf, 0x99a2b6ca, 0x0, 0x21a2b2c9, 0xf99fb5c8, 0xffdce0e4, 0xfff5f4fa, 0xfff5f4fb, 
0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 
0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xff9f9ea2, 0xff585859, 0xffb7b6bb, 0xfff3f2f8, 
0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 
0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff5f4fb, 0xfff7f6fc, 
0xfffdfdfe, 0xffb2c1ce, 0x4ea0b4c7, 0x789db3c6, 0xffc0cad4, 0xfff4f4fa, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 
0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 
0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff3f3fa, 0xff86868a, 0xff3b3b3b, 0xff4e4e4e, 0xffa6a6ab, 0xffededf3, 0xfff4f4fb, 
0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 
0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff4f4fb, 0xfff6f6fc, 0xffd1d7df, 
0x9f9fb4c5, 0xff9cb2c5, 0xffe8e9ed, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 
0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 
0xfff4f3fa, 0xffe3e2e9, 0xff656467, 0xff2c2c2c, 0xff464646, 0xff484848, 0xff969599, 0xffe8e7ed, 0xfff4f3fa, 0xfff4f3fa, 
0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 
0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff4f3fa, 0xfff1f1f5, 0xff9cb2c5, 0xff9ab0c4, 
0xfff1f0f5, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 
0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xffd3d2d8, 
0xff4a4a4c, 0xff262626, 0xff3a3a3a, 0xff4c4c4c, 0xff434343, 0xff828285, 0xffe3e2e8, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 
0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 
0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff3f2f9, 0xfff2f1f6, 0xff9ab0c4, 0xff99afc2, 0xfff1f1f5, 0xfff2f2f9, 
0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 
0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xffb8b8bd, 0xff343435, 0xff242424, 
0xff323232, 0xff474747, 0xff4e4e4e, 0xff414141, 0xff747477, 0xffd7d7de, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 
0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 
0xfff2f2f9, 0xfff2f2f9, 0xfff2f2f9, 0xfff1f1f6, 0xff99afc2, 0xff97adc0, 0xfff0f0f4, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 
0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 
0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xff949498, 0xff2c2c2c, 0xff262626, 0xff2d2d2d, 0xff3d3d3d, 
0xff4d4d4d, 0xff4c4c4c, 0xff3f3f3f, 0xff6a6a6d, 0xffd1d1d7, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 
0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 0xfff1f1f8, 
0xfff1f1f8, 0xfff0f0f5, 0xff97adc0, 0xff95abbf, 0xfff0eff4, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 
0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 
0xfff1f0f8, 0xfff1f0f8, 0xffe7e6ed, 0xff6e6e71, 0xff242424, 0xff262626, 0xff2a2a2a, 0xff373737, 0xff474747, 0xff505050, 
0xff484848, 0xff3e3e3e, 0xff646366, 0xffd2d1d8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 
0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff1f0f8, 0xfff0eff5, 
0xff95abbf, 0xff93a9bd, 0xffefeef3, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 
0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 
0xffcecdd4, 0xff48484a, 0xff222222, 0xff262626, 0xff2a2a2a, 0xff343434, 0xff414040, 0xff4c4c4c, 0xff4f4e4e, 0xff434343, 
0xff3c3c3c, 0xff6d6d70, 0xffdddce3, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 
0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xfff0eff7, 0xffefeef4, 0xff93a9bd, 0xff91a7bb, 
0xffeeeef2, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 
0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffa4a4a9, 0xff2f2f30, 
0xff242424, 0xff272727, 0xff2b2a2a, 0xff2f3030, 0xff373c3c, 0xff424848, 0xff4e5050, 0xff4a4949, 0xff3e3d3d, 0xff383838, 
0xff87878b, 0xffeaeaf1, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 
0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffefeff6, 0xffeeeef3, 0xff91a7bb, 0xff8ea5b9, 0xffededf1, 0xffeeeef5, 
0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 
0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffe4e4eb, 0xff666669, 0xff232323, 0xff242424, 0xff272626, 
0xff222929, 0xff222e2e, 0xff2f3737, 0xff3b4141, 0xff414c4c, 0xff444d4d, 0xff3f4040, 0xff393939, 0xff3e3e3e, 0xffadadb2, 
0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 
0xffeeeef5, 0xffeeeef5, 0xffeeeef5, 0xffededf2, 0xff8ea5b9, 0xff8ca3b7, 0xffececf0, 0xffededf4, 0xffededf4, 0xffededf4, 
0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 
0xffededf4, 0xffededf4, 0xffededf4, 0xffbababf, 0xff363637, 0xff242323, 0xff232626, 0xff1b2626, 0xff2a2828, 0xff5c3030, 
0xff8c3a3a, 0xff984040, 0xff7d4343, 0xff544646, 0xff394444, 0xff343838, 0xff393838, 0xff575759, 0xffd4d4da, 0xffededf4, 
0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 0xffededf4, 
0xffededf4, 0xffececf1, 0xff8ca3b7, 0xff8aa1b5, 0xffebebf0, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 
0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 
0xffe8e8ef, 0xff76767a, 0xff262425, 0xff222525, 0xff182525, 0xff3e2525, 0xffa03232, 0xffeb4242, 0xffff4c4c, 0xffff4f4f, 
0xfffa4949, 0xffc54343, 0xff6c3e3e, 0xff323b3b, 0xff333636, 0xff363535, 0xff8f8f93, 0xffe9e9f1, 0xffececf4, 0xffececf4, 
0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffececf4, 0xffebebf1, 
0xff8aa1b5, 0xff879fb3, 0xffeaeaef, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 
0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffbebec4, 0xff404041, 
0xff222323, 0xff182424, 0xff372323, 0xffb32e2e, 0xffff4141, 0xffff4c4c, 0xffff5151, 0xffff5555, 0xffff5555, 0xffff5050, 
0xffe34141, 0xff693838, 0xff2c3535, 0xff353737, 0xff4b4a4b, 0xffccccd3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 
0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffebebf3, 0xffeaeaf0, 0xff879fb3, 0xff859db1, 
0xffe9e9ee, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 
0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffe6e6ee, 0xff7a7a7e, 0xff262626, 0xff1b2424, 0xff212121, 
0xff7a2424, 0xffe53636, 0xffff4141, 0xffff4747, 0xffff5050, 0xffff5a5a, 0xffff5a5a, 0xffff5454, 0xffff4c4c, 0xffcc3939, 
0xff473232, 0xff2a3434, 0xff343434, 0xff909095, 0xffe8e8f0, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 
0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffeaeaf2, 0xffe9e9ef, 0xff859db1, 0xff829baf, 0xffe8e8ed, 0xffe9e9f1, 
0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 
0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffc6c6cd, 0xff424143, 0xff212323, 0xff192323, 0xff361d1d, 0xff8e2222, 0xffcc3232, 
0xffec3b3b, 0xfffb4040, 0xffff4b4b, 0xffff5656, 0xffff5757, 0xffff4e4e, 0xffff4949, 0xfffc3c3c, 0xff752f2f, 0xff2b3232, 
0xff333737, 0xff4e4d4f, 0xffd7d7de, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 
0xffe9e9f1, 0xffe9e9f1, 0xffe9e9f1, 0xffe8e8ee, 0xff829baf, 0xff8098ac, 0xffe6e6ec, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 
0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 
0xffe4e4ed, 0xff7f7f83, 0xff1d1d1d, 0xff1c2020, 0xff1b2020, 0xff3b1818, 0xff7d1d1d, 0xffa92929, 0xffc83232, 0xffe33838, 
0xfff43f3f, 0xfffa4444, 0xfffd4646, 0xfffc4242, 0xfff83f3f, 0xfff43b3b, 0xff892929, 0xff201f1f, 0xff181f1f, 0xff201f20, 
0xffa3a3a9, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 0xffe7e7f0, 
0xffe7e7f0, 0xffe6e6ed, 0xff8098ac, 0xff7d96aa, 0xffe6e6eb, 0xffe6e6ef, 0xffe6e6ef, 0xffe6e6ef, 0xffe6e6ef, 0xffe6e6ef, 
0xffe6e6ef, 0xffe6e6ef, 0xffe6e6ef, 0xffd0d0d8, 0xffbdbdc4, 0xffadadb3, 0xff9b9ba1, 0xff88888d, 0xff707073, 0xff313233, 
0xff151313, 0xff131616, 0xff101212, 0xff2d0e0e, 0xff651818, 0xff8a2323, 0xffa72a2a, 0xffc23131, 0xffd53535, 0xffe03737, 
0xffe63939, 0xffe43838, 0xffdb3737, 0xffd03030, 0xff882a2a, 0xff5b5555, 0xff6d7373, 0xff7d7d7d, 0xff88888b, 0xffe6e6ef, 
0xffe6e6ef, 0xffe6e6ef, 0xffe6e6ef, 0xffe6e6ef, 0xffd9d9e1, 0xffb8b8bf, 0xffcfcfd7, 0xffe6e6ef, 0xffe6e6ef, 0xffe6e6ec, 
0xff7d96aa, 0xff7b94a8, 0xffe5e5ea, 0xffe5e5ee, 0xffe5e5ee, 0xffe5e5ee, 0xffe5e5ee, 0xffe5e5ee, 0xffe5e5ee, 0xffb0b0b7, 
0xff767679, 0xff5b5b5d, 0xff4e4d4e, 0xff454546, 0xff383839, 0xff2e2e2e, 0xff393939, 0xff6a6969, 0xff8a8a8a, 0xff858787, 
0xff5f6161, 0xff301b1b, 0xff420909, 0xff6b1a1a, 0xff862222, 0xffa02929, 0xffb32d2d, 0xffbe2f2f, 0xffc43030, 0xffc13030, 
0xffbc3131, 0xffa01919, 0xff8c4040, 0xffcec6c6, 0xfff0f1f1, 0xfff7f7f7, 0xffc9c9ca, 0xffb2b2b8, 0xffe5e5ee, 0xffe5e5ee, 
0xffe5e5ee, 0xffe3e3ec, 0xff9c9ca2, 0xff5d5d5f, 0xff808083, 0xffd8d8e1, 0xffe4e4ed, 0xffe5e5eb, 0xff7b94a8, 0xff7891a6, 
0xffe4e4e9, 0xffe4e4ed, 0xffe4e4ed, 0xffe4e4ed, 0xffe4e4ed, 0xffc7c7cf, 0xff7e7b7e, 0xff3d3836, 0xff23201c, 0xff1b1814, 
0xff272320, 0xff191615, 0xff0f0e0e, 0xff0c0c0c, 0xff424242, 0xffacabab, 0xffdbdbdb, 0xffe1e3e3, 0xffb4b7b7, 0xff443a3a, 
0xff210000, 0xff490f0f, 0xff651b1b, 0xff7d2020, 0xff922424, 0xff9e2828, 0xffa12828, 0xff9d2727, 0xff9a2929, 0xff6e0606, 
0xff895858, 0xfff1eded, 0xffffffff, 0xffffffff, 0xfff1f1f1, 0xff676768, 0xff848489, 0xffe2e2eb, 0xffe4e4ed, 0xffe0e0e9, 
0xff7e7e83, 0xff363636, 0xff515153, 0xffbcbcc4, 0xffe2e2eb, 0xffe4e4ea, 0xff7891a6, 0xff758fa3, 0xffe3e3e8, 0xffe3e3ec, 
0xffe3e3ec, 0xffd8d8e0, 0xff919095, 0xff4b4947, 0xff3c4144, 0xff5a6e7c, 0xff66849b, 0xff738da3, 0xff81909f, 0xff8c9096, 
0xff8c8c91, 0xff535356, 0xff5f6060, 0xffb7b7b7, 0xffcdcdcd, 0xffcccccc, 0xffabadad, 0xff494646, 0xff100202, 0xff270707, 
0xff421111, 0xff551616, 0xff6b1a1a, 0xff7a1f1f, 0xff7b1f1f, 0xff761e1e, 0xff6b1717, 0xff3b0000, 0xff978383, 0xfff8f9f9, 
0xffffffff, 0xffffffff, 0xfff5f5f5, 0xff818181, 0xff2e2e2f, 0xffa8a8af, 0xffe3e3ec, 0xffe2e2eb, 0xff909095, 0xff0e0e0e, 
0xff39393a, 0xffb7b7be, 0xffe0e0e9, 0xffe3e3e9, 0xff758fa3, 0xff738da1, 0xffe1e2e7, 0xffd4d5de, 0xff9e9fa5, 0xff616163, 
0xff363535, 0xff576674, 0xff88b0d4, 0xff94c5f0, 0xff91c5ff, 0xff96c9ff, 0xffa3d2f7, 0xffa1c2d9, 0xffacb5bf, 0xffb2b4ba, 
0xff878687, 0xffbababa, 0xffc3c3c3, 0xffbab9b9, 0xff989999, 0xff505050, 0xff100b0b, 0xff0c0000, 0xff210707, 0xff2f0c0c, 
0xff3f0f0f, 0xff4c1313, 0xff4e1414, 0xff4c1414, 0xff280202, 0xff362020, 0xffc9c8c8, 0xffffffff, 0xffffffff, 0xffffffff, 
0xfffcfcfc, 0xffb0b0b0, 0xff262626, 0xff66666a, 0xffe0e1ea, 0xffe1e2eb, 0xffa0a0a7, 0xff070707, 0xff3a3a3c, 0xffc7c7cf, 
0xffdfe0e9, 0xffe1e2e8, 0xff738da1, 0xff708a9f, 0xffe0e1e6, 0xffcacad3, 0xff3c3a3b, 0xff262627, 0xff546875, 0xff9ac6ed, 
0xff9accff, 0xff94c8ff, 0xff91c9ff, 0xff94ccff, 0xff9bd5ff, 0xffa6e1ff, 0xff97c4e6, 0xff7d8b95, 0xff898684, 0xffbab8b7, 
0xffbdbebe, 0xffb1b1b1, 0xff919090, 0xff575858, 0xff222323, 0xff0c0707, 0xff0b0000, 0xff120202, 0xff180606, 0xff1e0606, 
0xff210606, 0xff150202, 0xff0a0202, 0xff787777, 0xfff1f1f1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffdbdbdb, 
0xff3a3a3a, 0xff3d3d3f, 0xffc3c4cc, 0xffe0e1ea, 0xff9b9ca2, 0xff212121, 0xff545456, 0xffc9c9d1, 0xffdedfe7, 0xffe0e1e7, 
0xff708a9f, 0xff6b859a, 0xffdedee4, 0xffacacb3, 0xff282828, 0xff363737, 0xff6a7e8b, 0xffa3d2fd, 0xff99ccff, 0xff93c4f4, 
0xffa6d2e7, 0xffabd1de, 0xff8bb0d0, 0xff7593b5, 0xff76859a, 0xff7d7f84, 0xff979493, 0xffb5b4b4, 0xffb8b9b9, 0xffababab, 
0xff8f8f8f, 0xff5b5c5c, 0xff2e3030, 0xff212121, 0xff171414, 0xff0d0606, 0xff080101, 0xff090101, 0xff090202, 0xff0a0707, 
0xff404040, 0xffb8baba, 0xfffcfcfc, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffebebeb, 0xff565656, 0xff262627, 
0xff98989f, 0xffdcdce5, 0xff6e6e72, 0xff333333, 0xff535354, 0xffa3a3aa, 0xffdbdbe5, 0xffdfdfe6, 0xff6b859a, 0xff688397, 
0xffdadadf, 0xff6c6c70, 0xff242424, 0xff2c2c2c, 0xff37393c, 0xff60707f, 0xff72899e, 0xff9099a8, 0xffc7cbcf, 0xffd6d9da, 
0xffb4b0b6, 0xff8d848b, 0xff8d8383, 0xff938f88, 0xff9f9e9c, 0xffb1b2b3, 0xffb5b4b5, 0xffa8a8a8, 0xff929292, 0xff696969, 
0xff3a3a3a, 0xff272828, 0xff282929, 0xff252424, 0xff1f1d1d, 0xff1a1919, 0xff1c1c1c, 0xff393939, 0xff888888, 0xffdfdfdf, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfff2f2f2, 0xff787878, 0xff121212, 0xff606063, 0xff7e7e83, 
0xff2f2f2f, 0xff3c3c3c, 0xff4c4c4c, 0xff707073, 0xffb4b4bc, 0xffdedee4, 0xff688397, 0xff658095, 0xffccccd1, 0xff414244, 
0xff242424, 0xff302f2f, 0xff32312d, 0xff1f1f1e, 0xff373736, 0xff756f6d, 0xffe2d8d5, 0xfffdf5ef, 0xffcac1ba, 0xff9a9389, 
0xff867c6e, 0xff8d8375, 0xffa3a1a0, 0xffb2b3b5, 0xffb3b3b2, 0xffa9a9a9, 0xff979797, 0xff7d7d7d, 0xff565656, 0xff373636, 
0xff2c2d2d, 0xff292929, 0xff282828, 0xff303030, 0xff4c4c4c, 0xff7e7e7e, 0xffc1c1c1, 0xfff5f5f5, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xfff9f9f9, 0xff9d9e9e, 0xff252627, 0xff2d2d2e, 0xff2b2c2d, 0xff111111, 0xff282828, 
0xff343434, 0xff4e4e50, 0xff8e8f94, 0xffdcdde3, 0xff658095, 0xff637e93, 0xffbbbcc0, 0xff2b2b2d, 0xff252626, 0xff413e3a, 
0xff4b4232, 0xff252017, 0xff0f0c0b, 0xff1b1b18, 0xff3f3e3b, 0xff706c64, 0xff8f8572, 0xff6d5f4a, 0xff6d5d46, 0xff958a7d, 
0xffa8a8a8, 0xffb0b1b2, 0xffb3b2b3, 0xffacacac, 0xff9f9f9f, 0xff8e8e8e, 0xff767676, 0xff5c5c5c, 0xff4d4d4d, 0xff494949, 
0xff535353, 0xff696969, 0xff8a8a8a, 0xffb6b6b6, 0xffe4e4e4, 0xfffdfdfd, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffbcbcbd, 0xff5f5f62, 0xff111112, 0xff141414, 0xff101010, 0xff1c1c1c, 0xff242424, 0xff464648, 
0xff84858a, 0xffdbdce2, 0xff637e93, 0xff607c90, 0xffbbbcc0, 0xff28292b, 0xff212122, 0xff45433f, 0xff5b4d3b, 0xff5a4b3a, 
0xff443a2d, 0xff171410, 0xff010100, 0xff090908, 0xff423b2c, 0xff79664b, 0xff84735c, 0xff9e958b, 0xffa8a7aa, 0xffb2b2b3, 
0xffb5b5b4, 0xffafafaf, 0xffaaaaaa, 0xffa0a0a0, 0xff929292, 0xff848484, 0xff7b7b7b, 0xff7c7c7c, 0xff888888, 0xff9b9b9b, 
0xffb5b5b5, 0xffd7d7d7, 0xfff6f6f6, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffefefe, 
0xffc8c8c9, 0xff8f9095, 0xff424245, 0xff060606, 0xff161616, 0xff2d2d2d, 0xff3a3a3a, 0xff4e4e50, 0xff8b8b91, 0xffdadbe1, 
0xff607c90, 0xff5d798e, 0xffcccdd2, 0xff414144, 0xff1c1c1c, 0xff3f3e3c, 0xff5e5344, 0xff70614b, 0xff5b4f40, 0xff0d0e0b, 
0xff000001, 0xff000606, 0xff171a16, 0xff4b4437, 0xff6c6459, 0xff91908d, 0xffabacac, 0xffb3b3b3, 0xffb6b6b6, 0xffb6b6b6, 
0xffb3b3b3, 0xffaeaeae, 0xffa8a8a8, 0xffa2a2a2, 0xffa0a0a0, 0xffa2a2a2, 0xffaaaaaa, 0xffbababa, 0xffd3d3d3, 0xffececec, 
0xfffdfdfd, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffcfcfc, 0xffc2c2c2, 0xff8f8f94, 
0xffbdbec6, 0xff434446, 0xff070707, 0xff111111, 0xff191919, 0xff505053, 0xffa3a4aa, 0xffd9dae0, 0xff5d798e, 0xff5b778c, 
0xffd3d4da, 0xff67686c, 0xff141414, 0xff2b2b2c, 0xff514d44, 0xff574e40, 0xff39362f, 0xff080e10, 0xff0c1315, 0xff141b1d, 
0xff141b1e, 0xff1e2327, 0xff4c4e51, 0xff949493, 0xffadadac, 0xffb2b2b2, 0xffb7b7b7, 0xffb9b9b9, 0xffb9b9b9, 0xffb7b7b7, 
0xffb6b6b6, 0xffb7b7b7, 0xffb7b7b7, 0xffbababa, 0xffc3c3c3, 0xffd1d1d1, 0xffe4e4e4, 0xfff8f8f8, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfff9f9f9, 0xffbababa, 0xff7e7e82, 0xffd7d8e2, 0xffa5a6ae, 
0xff161616, 0xff131313, 0xff1b1b1b, 0xff76767c, 0xffc4c5ce, 0xffd7d8df, 0xff5b778c, 0xff58758a, 0xffd5d5dc, 0xffa0a1a8, 
0xff101010, 0xff191819, 0xff2e2e2f, 0xff1a1f1f, 0xff171f22, 0xff3d4044, 0xff616367, 0xff818289, 0xffa2a3ab, 0xff95979d, 
0xff6e6d70, 0xff9a9a9a, 0xffaaaaaa, 0xffb1b1b1, 0xffb7b7b7, 0xffb9b9b9, 0xffbcbcbc, 0xffbdbdbd, 0xffbebebe, 0xffc2c2c2, 
0xffc6c6c6, 0xffcbcbcb, 0xffd2d2d2, 0xffdddddd, 0xffededed, 0xfffbfbfb, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xffffffff, 0xffffffff, 0xfff9f9f9, 0xffb6b6b6, 0xff737375, 0xffc7c8d1, 0xffc8c9d2, 0xff59595d, 0xff151516, 
0xff454548, 0xffb4b5bd, 0xffd3d4de, 0xffd7d7df, 0xff58758a, 0xff567287, 0xffd6d6dd, 0xffcdced7, 0xff66666b, 0xff0e0e0e, 
0xff1e1e1e, 0xff5c5d61, 0xffc4c5ce, 0xffcfd0da, 0xffd4d5df, 0xffd4d5df, 0xffd5d6e0, 0xffa9aab2, 0xff727173, 0xff9b9b9b, 
0xffa8a8a8, 0xffb0b0b0, 0xffb5b5b5, 0xffb8b8b8, 0xffbbbbbb, 0xffbebebe, 0xffc1c1c1, 0xffc5c5c5, 0xffcbcbcb, 0xffd1d1d1, 
0xffdadada, 0xffe4e4e4, 0xfff1f1f1, 0xfffcfcfc, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 
0xffffffff, 0xfff9f9f9, 0xffbababa, 0xff69696a, 0xffaeafb7, 0xffd5d6e0, 0xffd0d1db, 0xffc0c1ca, 0xffc8c9d2, 0xffd5d6e0, 
0xffd5d6e0, 0xffd6d6de, 0xff567287, 0xff537085, 0xffd5d5dc, 0xffd4d5df, 0xffbebfc8, 0xff444447, 0xff5a5a5e, 0xffc7c8d1, 
0xffd4d5df, 0xffd4d5df, 0xffd4d5df, 0xffd4d5df, 0xffd4d5df, 0xffa0a1a8, 0xff69696a, 0xff989898, 0xffa3a3a3, 0xffababab, 
0xffb0b0b0, 0xffb4b4b4, 0xffb7b7b7, 0xffbbbbbb, 0xffbfbfbf, 0xffc5c5c5, 0xffcacaca, 0xffd1d1d1, 0xffdadada, 0xffe5e5e5, 
0xfff0f0f0, 0xfff8f8f8, 0xfffdfdfd, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffbfbfb, 
0xffc4c4c4, 0xff6c6c6d, 0xffa0a1a8, 0xffd4d5df, 0xffd4d5df, 0xffd4d5df, 0xffd4d5df, 0xffd4d5df, 0xffd4d5df, 0xffd5d5dd, 
0xff537085, 0xff516e83, 0xffd4d5db, 0xffd2d3dd, 0xffd2d3dd, 0xffcccdd7, 0xffcecfd9, 0xffd3d4de, 0xffd1d2dc, 0xffd1d2dc, 
0xffd1d2db, 0xffd3d4de, 0xffd3d4de, 0xff929399, 0xff6b6b6c, 0xff949494, 0xff9c9c9c, 0xffa4a4a4, 0xffa9a9a9, 0xffadadad, 
0xffb1b1b1, 0xffb4b4b4, 0xffb9b9b9, 0xffc0c0c0, 0xffc5c5c5, 0xffcccccc, 0xffd4d4d4, 0xffdddddd, 0xffe6e6e6, 0xffededed, 
0xfff3f3f3, 0xfff9f9f9, 0xfffdfdfd, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfffcfcfc, 0xffcfcfcf, 0xff757576, 
0xff95969c, 0xffd3d4de, 0xffd3d4de, 0xffd3d4de, 0xffd3d4de, 0xffd3d4de, 0xffd3d4de, 0xffd4d5dc, 0xff516e83, 0xff4e6c81, 
0xffd3d4da, 0xffd1d2dc, 0xffd0d1db, 0xffd2d3dd, 0xffcfd0da, 0xffcbccd5, 0xffcacbd4, 0xffc9cad3, 0xffc7c8d2, 0xffcbccd5, 
0xffd0d1db, 0xff8d8e94, 0xff656566, 0xff8d8d8d, 0xff959595, 0xff9b9b9b, 0xff9f9f9f, 0xffa2a2a2, 0xffa6a6a6, 0xffaaaaaa, 
0xffaeaeae, 0xffb4b4b4, 0xffbbbbbb, 0xffc2c2c2, 0xffc8c8c8, 0xffd0d0d0, 0xffd7d7d7, 0xffdcdcdc, 0xffe4e4e4, 0xffededed, 
0xfff3f3f3, 0xfffafafa, 0xfffdfdfd, 0xffffffff, 0xffffffff, 0xfffdfdfd, 0xffd9d9d9, 0xff89898a, 0xff8f8f96, 0xffd2d3dd, 
0xffd2d3dd, 0xffd2d3dd, 0xffd2d3dd, 0xffd2d3dd, 0xffd2d3dd, 0xffd3d4db, 0xff4e6c81, 0xff4c6a7f, 0xffd3d3da, 0xffd1d2dc, 
0xffcfd0d9, 0xffc8c9d3, 0xffc3c4cd, 0xffbebfc8, 0xffbcbdc6, 0xffb9bac3, 0xffb4b5be, 0xffb5b6bf, 0xffb9bac3, 0xff818187, 
0xff5f5f60, 0xff848484, 0xff888888, 0xff8e8e8e, 0xff919191, 0xff949494, 0xff979797, 0xff9b9b9b, 0xff9e9e9e, 0xffa4a4a4, 
0xffababab, 0xffb0b0b0, 0xffb5b5b5, 0xffbcbcbc, 0xffc4c4c4, 0xffcacaca, 0xffd2d2d2, 0xffdbdbdb, 0xffe3e3e3, 0xffebebeb, 
0xfff3f3f3, 0xfffafafa, 0xfffdfdfd, 0xfff8f8f8, 0xffdcdcdc, 0xff979798, 0xff929399, 0xffd1d2dc, 0xffd1d2dc, 0xffd1d2dc, 
0xffd1d2dc, 0xffd1d2dc, 0xffd1d2dc, 0xffd2d3da, 0xff4c6a7f, 0xff4a687d, 0xffd2d4db, 0xffced0db, 0xffc9cad5, 0xffbdbfc9, 
0xffb3b5bf, 0xffadafb8, 0xffa8aab3, 0xffa2a4ad, 0xff9c9da6, 0xff999ba3, 0xff9b9da5, 0xff717278, 0xff5b5b5c, 0xff7d7d7d, 
0xff7d7d7d, 0xff7e7e7e, 0xff7e7e7e, 0xff7f7f7f, 0xff828282, 0xff858585, 0xff888888, 0xff8d8d8d, 0xff959595, 0xff979797, 
0xff9c9c9c, 0xffa3a3a3, 0xffababab, 0xffb3b3b3, 0xffbbbbbb, 0xffc5c5c5, 0xffcfcfcf, 0xffdadada, 0xffe3e3e3, 0xffececec, 
0xfff3f3f3, 0xffeeeeee, 0xffdbdbdb, 0xff9b9b9c, 0xff97989f, 0xffcfd1dc, 0xffcfd1dc, 0xffcfd1dc, 0xffcfd1dc, 0xffcfd1dc, 
0xffcfd1dc, 0xffd1d3db, 0xff4a687d, 0xff47667b, 0xffd0d1d8, 0xffc3c5d0, 0xffb6b8c1, 0xffaaabb4, 0xffa0a2aa, 0xff999aa2, 
0xff909199, 0xff86878f, 0xff7e7f86, 0xff7c7d83, 0xff7c7e84, 0xff626267, 0xff575758, 0xff7a7a7a, 0xff767676, 0xff6d6d6d, 
0xff686868, 0xff666666, 0xff656565, 0xff666666, 0xff686868, 0xff6d6d6d, 0xff747474, 0xff777777, 0xff7c7c7c, 0xff838383, 
0xff8b8b8b, 0xff939393, 0xff9c9c9c, 0xffaaaaaa, 0xffb6b6b6, 0xffc4c4c4, 0xffd1d1d1, 0xffdfdfdf, 0xffe8e8e8, 0xffe9e9e9, 
0xffdadada, 0xff9b9b9c, 0xff9fa1a8, 0xffced0db, 0xffced0db, 0xffced0db, 0xffced0db, 0xffced0db, 0xffced0db, 0xffd1d2da, 
0xff47667b, 0xff456479, 0xffc6c7ce, 0xffafb0ba, 0xffa1a2aa, 0xff9798a0, 0xff8d8e96, 0xff85868d, 0xff797a80, 0xff6c6d73, 
0xff636368, 0xff5e5e63, 0xff5a5a5f, 0xff46474a, 0xff4c4c4d, 0xff777777, 0xff757575, 0xff656565, 0xff565656, 0xff4d4d4d, 
0xff484848, 0xff464646, 0xff464646, 0xff494949, 0xff4f4f4f, 0xff525252, 0xff565656, 0xff5c5c5c, 0xff636363, 0xff6d6d6d, 
0xff787878, 0xff878787, 0xff989898, 0xffadadad, 0xffc1c1c1, 0xffd3d3d3, 0xffe0e0e0, 0xffe4e4e4, 0xffd8d8d8, 0xff959596, 
0xffa5a6ae, 0xffcecfda, 0xffcecfda, 0xffcecfda, 0xffcecfda, 0xffcecfda, 0xffcecfda, 0xffd1d2da, 0xff456479, 0xff436277, 
0xffb8b9bf, 0xff9c9da5, 0xff909198, 0xff85868d, 0xff7a7b81, 0xff6e6f75, 0xff5d5e63, 0xff4e4e53, 0xff444548, 0xff3e3e42, 
0xff363639, 0xff2a2a2c, 0xff373738, 0xff6b6b6b, 0xff767676, 0xff666666, 0xff525252, 0xff414141, 0xff363636, 0xff313131, 
0xff303030, 0xff2f2f2f, 0xff323232, 0xff333334, 0xff353536, 0xff393939, 0xff3f3f3f, 0xff464646, 0xff525252, 0xff636363, 
0xff797979, 0xff949494, 0xffb0b0b0, 0xffc8c8c8, 0xffd8d8d8, 0xffdbdbdb, 0xffcecece, 0xff868688, 0xffacadb6, 0xffcdced9, 
0xffcccdd8, 0xffcdced9, 0xffcdced9, 0xffcdced9, 0xffcdced9, 0xffd1d2da, 0xff436277, 0xff416076, 0xffafb0b6, 0xff909198, 
0xff85858d, 0xff7a7b82, 0xff727278, 0xff68696e, 0xff5a5b60, 0xff505055, 0xff49494d, 0xff454549, 0xff3d3d40, 0xff303033, 
0xff282829, 0xff505050, 0xff717171, 0xff686868, 0xff565656, 0xff434343, 0xff373737, 0xff2f2f2f, 0xff2c2c2c, 0xff2a2a2b, 
0xff282829, 0xff272728, 0xff262627, 0xff282829, 0xff2d2d2d, 0xff303031, 0xff373737, 0xff454545, 0xff5b5b5b, 0xff797979, 
0xff9a9a9a, 0xffb6b6b6, 0xffc9c9c9, 0xffcccccc, 0xffb7b7b7, 0xff737476, 0xffb3b4be, 0xffc4c5d0, 0xffc4c5d0, 0xffcacbd5, 
0xffcccdd8, 0xffcccdd8, 0xffcccdd8, 0xffd1d1d9, 0xff416076, 0xff3f5e74, 0xffb9bac0, 0xffa0a2aa, 0xff989aa2, 0xff93959d, 
0xff91929a, 0xff8e8f97, 0xff898a92, 0xff87888f, 0xff82838a, 0xff797a81, 0xff68696f, 0xff4e4f53, 0xff2a2b2d, 0xff222222, 
0xff484848, 0xff535353, 0xff4c4c4c, 0xff3f3f3f, 0xff333333, 0xff282828, 0xff252526, 0xff29292b, 0xff303133, 0xff353639, 
0xff353538, 0xff2e2f31, 0xff272729, 0xff252526, 0xff282828, 0xff343434, 0xff444444, 0xff5e5e5e, 0xff808080, 0xffa2a2a2, 
0xffb5b5b5, 0xffb7b7b7, 0xff939393, 0xff717277, 0xffb7b9c3, 0xffb5b6c0, 0xffb9bbc5, 0xffc1c3cd, 0xffc7c9d4, 0xffcbcdd8, 
0xffcbcdd8, 0xffd0d2da, 0xff3f5e74, 0xff3e5d72, 0xffc2c5cc, 0xffc0c2cc, 0xffbdbfca, 0xffbcbec8, 0xffbcbec8, 0xffbcbec8, 
0xffbdbec9, 0xffbec0ca, 0xffb9bbc5, 0xffb1b2bc, 0xffa1a2ab, 0xff84868d, 0xff58595e, 0xff242527, 0xff19191a, 0xff262627, 
0xff2f2f30, 0xff2c2c2d, 0xff262728, 0xff292a2c, 0xff38393c, 0xff4c4d51, 0xff5d5e63, 0xff66676d, 0xff64656a, 0xff56565b, 
0xff3f3f43, 0xff242527, 0xff171818, 0xff1c1c1c, 0xff2c2c2c, 0xff434343, 0xff5e5e5e, 0xff7e7e7e, 0xff909090, 0xff868686, 
0xff5d5d5f, 0xff8c8e95, 0xffb3b5bf, 0xffb2b4be, 0xffb4b6bf, 0xffb9bbc5, 0xffc0c2cc, 0xffc7c9d4, 0xffc8cad5, 0xffc8cbd3, 
0xff3e5d72, 0x783c5b71, 0xff8998a3, 0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 0xffc9cbd7, 
0xffc8cad6, 0xffc4c6d2, 0xffc0c1cd, 0xffb8bac5, 0xffa8aab4, 0xff8d8e97, 0xff75767d, 0xff696a70, 0xff68696f, 0xff696a70, 
0xff6d6e74, 0xff75767d, 0xff7f8088, 0xff8c8e96, 0xff9a9ba4, 0xffa1a2ac, 0xff9e9fa9, 0xff8f9099, 0xff72737a, 0xff4f5054, 
0xff2f2f32, 0xff141415, 0xff0f0f0f, 0xff202020, 0xff323232, 0xff404040, 0xff434343, 0xff3a3b3c, 0xff68696f, 0xffadaeb9, 
0xffb5b7c2, 0xffb8bac5, 0xffbabcc7, 0xffbcbeca, 0xffc1c3cf, 0xffc5c7d3, 0xffc7c9d5, 0xff8b9aa5, 0x783c5b71, 0x213e5d6c, 
0xf9426176, 0xffc5c8d1, 0xffc9cbd6, 0xffc9cbd6, 0xffc9cbd6, 0xffc9cbd6, 0xffc9cbd6, 0xffc9cbd6, 0xffc8cad5, 0xffc7c9d3, 
0xffc4c6d1, 0xffc4c6d1, 0xffc7c9d3, 0xffc9cbd6, 0xffc9cbd6, 0xffc5c7d2, 0xffc1c3ce, 0xffbdbfc9, 0xffbbbdc7, 0xffbbbdc7, 
0xffbdbfc9, 0xffc0c2cd, 0xffc3c5d0, 0xffc5c7d2, 0xffc3c5d0, 0xffbec0ca, 0xffb1b2bc, 0xff9e9fa8, 0xff85878e, 0xff6a6b70, 
0xff505156, 0xff3c3d40, 0xff38383b, 0xff38393c, 0xff454549, 0xff75777d, 0xffb3b5bf, 0xffc1c3ce, 0xffc3c5d0, 0xffc5c7d2, 
0xffc6c8d3, 0xffc7c9d3, 0xffc7c9d4, 0xffc8cad5, 0xffc2c6cd, 0xf93d5c72, 0x213e5d6c, 0x0, 0x663a586e, 0xfc3a596f, 
0xffa4acb6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffc9cbd5, 0xffc8cad3, 
0xffc8cad4, 0xffc8cad4, 0xffc8cad4, 0xffc9cbd5, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 
0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffc4c6d0, 0xffbdbfc9, 0xffb1b3bc, 
0xffa8aab2, 0xffaaacb4, 0xffbbbdc6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 0xffcaccd6, 
0xffcaccd6, 0xff97a0aa, 0xfc3a596f, 0x663a586e, 0x0, 0x0, 0x0, 0x3337556e, 0x9636576d, 0xff37576d, 
0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 
0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 
0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 
0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0xff37576d, 0x9636576d, 
0x3337556e, 0x0, 0x0, }; }
