package org.omg.PortableInterceptor;


/**
* org/omg/PortableInterceptor/ACTIVE.java .
* Generated by the IDL-to-Java compiler (portable), version "3.2"
* from ../../../../src/share/classes/org/omg/PortableInterceptor/Interceptors.idl
* Friday, May 25, 2007 3:39:56 o'clock PM GMT-05:00
*/

public interface ACTIVE
{

  /** Object adapter state that indicates normal request processing.
     */
  public static final short value = (short)(1);
}
