package sun.awt.X11;
public class XAWTIcon64_security_icon_bw24_png {
public static long[] security_icon_bw24_png = { 
24,24,
0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0xffa1a1a1, 0xffa1a1a1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0xffa0a0a0, 0xffcbcbcb, 0xffcbcbcb, 0xffa0a0a0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff9e9e9e, 0xffe1e1e1, 0xffe1e1e1, 
0xff9e9e9e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0xff9d9d9d, 0xffcacaca, 0xffe0e0e0, 0xffe0e0e0, 0xffcacaca, 0xff9d9d9d, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0xff9b9b9b, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xffdfdfdf, 0xff9b9b9b, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff999999, 0xffc8c8c8, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffc8c8c8, 0xff999999, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff979797, 0xffdcdcdc, 0xffdcdcdc, 0xffb5b5b5, 0xffbfbfbf, 0xffdcdcdc, 0xffdcdcdc, 0xff979797, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0xff959595, 0xffc6c6c6, 0xffdbdbdb, 0xff939393, 0xff393939, 0xff393939, 
0xffbcbcbc, 0xffdbdbdb, 0xffc6c6c6, 0xff959595, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff929292, 0xffdadada, 
0xffdadada, 0xff838383, 0xff2d2d2d, 0xff2d2d2d, 0xffafafaf, 0xffdadada, 0xffdadada, 0xff929292, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0xff909090, 0xffc4c4c4, 0xffd9d9d9, 0xffd9d9d9, 0xff7d7d7d, 0xff222222, 0xff222222, 0xffababab, 0xffd9d9d9, 
0xffd9d9d9, 0xffc4c4c4, 0xff909090, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0xff8d8d8d, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffa8a8a8, 
0xff191919, 0xff191919, 0xffcccccc, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xff8d8d8d, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff8b8b8b, 0xffc2c2c2, 
0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffa5a5a5, 0xff111111, 0xff111111, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 0xffd7d7d7, 
0xffc2c2c2, 0xff8b8b8b, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0xff888888, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffbdbdbd, 0xff0b0b0b, 0xff181818, 
0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xffd6d6d6, 0xff888888, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0x0, 0x0, 0x0, 0xff858585, 0xffc3c3c3, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 
0xffd8d8d8, 0xffd8d8d8, 0xff060606, 0xff3b3b3b, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffd8d8d8, 0xffc3c3c3, 
0xff858585, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff838383, 
0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xff020202, 0xff383838, 0xffdadada, 0xffdadada, 
0xffdadada, 0xffdadada, 0xffdadada, 0xffdadada, 0xff838383, 0x0, 0x0, 0x0, 0x0, 0x0, 
0x0, 0x0, 0xff808080, 0xffc6c6c6, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 
0xff383838, 0xff6e6e6e, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffdcdcdc, 0xffc6c6c6, 0xff808080, 
0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff7e7e7e, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 0xffc2c2c2, 0xffd0d0d0, 0xffdedede, 0xffdedede, 0xffdedede, 0xffdedede, 
0xffdedede, 0xffdedede, 0xffdedede, 0xff7e7e7e, 0x0, 0x0, 0x0, 0x0, 0x0, 0xff7b7b7b, 
0xffcacaca, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffb9b9b9, 0xffc3c3c3, 
0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffe1e1e1, 0xffcacaca, 0xff7b7b7b, 0x0, 
0x0, 0x0, 0x0, 0xff797979, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 
0xffe4e4e4, 0xff9a9a9a, 0xff242424, 0xff242424, 0xffcbcbcb, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 0xffe4e4e4, 
0xffe4e4e4, 0xffe4e4e4, 0xff797979, 0x0, 0x0, 0x0, 0xff777777, 0xffcfcfcf, 0xffe7e7e7, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffb0b0b0, 0xff070707, 0xff151515, 0xffd9d9d9, 0xffe7e7e7, 
0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffe7e7e7, 0xffcfcfcf, 0xff777777, 0x0, 0x0, 
0xff757575, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 
0xffcdcdcd, 0xffdbdbdb, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 0xffeaeaea, 
0xffeaeaea, 0xff757575, 0x0, 0xff737373, 0xffd5d5d5, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 
0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 
0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffeeeeee, 0xffd5d5d5, 0xff737373, 0xff727272, 0xffc2c2c2, 0xffd7d7d7, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 
0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xfff1f1f1, 0xffd7d7d7, 0xffc2c2c2, 
0xff727272, 0x0, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 0xff707070, 
0xff707070, 0xff707070, 0xff707070, 0xff707070, 0x0, }; }
